/****************************************************************************/
/*  File Name:          AT17.C							                    */                                       
/*  Title:              UART and Two-wire header file                       */
/*  Last Updated:       August 5, 2001						                */
/*  Target Device:      AT94K05AL/10AL/40AL-25DQC (208-pin packet only)     */ 
/*												                            */
/*   Support Email:      fpslic@atmel.com						            */
/*   Support Hotline:    +1 (408) 436-4119 (USA)				            */
/****************************************************************************/

#ifndef __AT17_H
#define __AT17_H

#include <ioat94k.h>


/* Device Address Constants */
#define AT17LV010                       0xA6            /* Device Address Byte */
#define WRITE                           0
#define READ                            1

/* AT17LV010 Device Constants */
#define A_BYTE                          3               /* Bytes per Address */
#define R_BYTE                          4               /* Reset Polarity Locations */
#define T_BYTE                          128             /* Bytes per Page */
#define T_PAGE                          1024            /* Pages per Device */
#define TR_BYTE                         133             /* Bytes per each transmittion */

/* #define TT_BYTE                         131072       */         

#define LOW                             0x00            /* Reset Polarity */
#define HIGH                            0xFF            /* Reset Polarity */
#define raddress                        131072          /* first address location 
                                                            for reset polarity data */

/* Constant for Main program */

#define good							0x05            /* status = good packet*/
#define bad                             0x45            /* status = bad  packet*/
#define end                             0x50            /* status = end data transition*/
#define err                             0xAA            /* status = error packet */
#define dup                             0x09            /* status = dup packet */



/* UART Baud Rate for 4MHz Crystal (0.2% Error) */

#define BAUD1400     103
#define BAUD4800     51
#define BAUD9600     25
#define BAUD19K2     12


/* Constant signals used for transmission */ 
#define C_character  67
#define SOH          01
#define ACK          06
#define NAK          25
#define EOT          04

/* General Master Status Codes */
#define START                   0x08
#define REP_START               0x10

/* Master Transmitter Status Codes */
#define MT_SLA_ACK              0x18
#define MT_SLA_NACK             0x20
#define MT_DATA_ACK             0x28
#define MT_DATA_NACK            0x30
#define MT_ARB_LOST             0x38

/* Master Receiver Status Codes */
#define MR_ARB_LOST             0x38
#define MR_SLA_ACK              0x40
#define MR_SLA_NACK             0x48
#define MR_DATA_ACK             0x50
#define MR_DATA_NACK            0x58

/* Slave Receiver Status Codes */
#define SR_SLA_ACK              0x60
#define SR_ARB_LOST_SLA_ACK     0x68
#define SR_GCALL_ACK            0x70
#define SR_ARB_LOST_GCALL_ACK   0x78
#define SR_DATA_ACK             0x80
#define SR_DATA_NACK            0x88
#define SR_GCALL_DATA_ACK       0x90
#define SR_GCALL_DATA_NACK      0x98
#define SR_STOP                 0xA0

/* Slave Transmitter Status Codes */
#define ST_SLA_ACK              0xA8
#define ST_ARB_LOST_SLA_ACK     0xB0
#define ST_DATA_ACK             0xB8
#define ST_DATA_NACK            0xC0
#define ST_LAST_DATA            0xC8

/* Miscellaneous States */
#define NO_INFO                 0xF8
#define BUS_ERROR               0x00


/* TWS Macro Definitions */
#define CLEAR_TWINT()           (   TWCR    =  ( (1 << TWINT) | (1 << TWEN )               ) )
#define GET_BYTE(DATA)          ( ( DATA )  =  TWDR                                          )
#define REP_START_CONDITION()   (   TWCR    =  ( (1 << TWINT) | (1 << TWSTA) | (1 << TWEN) ) )
#define SEND_ACK()              (   TWCR    =  ( (1 << TWINT) | (1 << TWEA ) | (1 << TWEN) ) )
#define SEND_BYTE(DATA)         (   TWDR    =  ( DATA                                      ) )
#define START_CONDITION()       (   TWCR    =  ( (1 << TWSTA) | (1 << TWEN )               ) )
#define STOP_CONDITION()        (   TWCR    =  ( (1 << TWINT) | (1 << TWSTO) | (1 << TWEN) ) )

#define TWS_MASTER_INIT()       (   TWCR   &= ~( (1 << TWSTA) | (1 << TWSTO)               ) ); \
                                (   TWCR   |=  ( (1 << TWEN)                               ) )

#define TWS_SLAVE_INIT(ADDRESS) (   TWAR   &= ~( 0xFE                                      ) ); \
                                (   TWAR   |=  ( ADDRESS                                   ) ); \
                                (   TWCR   &= ~( (1 << TWSTA) | (1 << TWSTO)               ) ); \
                                (   TWCR    =  ( (1 << TWEA)  | (1 << TWEN)                ) )

#define WAIT()                  while( !( TWCR & (1 << TWINT) ) )




/* Function Prototypes */
void programResetPolarity(unsigned char newResetPolarityLevel, unsigned long resetAddress);
void writePage(unsigned char * data, unsigned long pageAddress);
void readPage(unsigned char * data, unsigned long pageAddress);
unsigned char verifyResetPolarity(unsigned long resetAddress);
int calcrc(unsigned char *ptr, int count);
void waitMS(unsigned char ms);
void initAT94K(void);
void initUART0(unsigned int baudRate);
unsigned char validate_packet(unsigned char *bufptr); 
void initTWS(unsigned char bitRate);
void respond(unsigned char packet);
unsigned char bit_swap(unsigned char *ptr);


static volatile unsigned char packet_number = 0;
static volatile unsigned long pageAddress =  0;




#endif /* __AT17_H */